#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import json

from omnix import util

from omnix.main import app
from omnix.main import flask
from omnix.main import quorum

@app.route("/entities", methods = ("GET",))
@quorum.ensure("foundation.root_entity.list")
def list_entities():
    return flask.render_template(
        "entity/list.html.tpl",
        link = "entities"
    )

@app.route("/entities.json", methods = ("GET",), json = True)
@quorum.ensure("foundation.root_entity.list")
def list_entities_json():
    api = util.get_api()
    object = quorum.get_object()
    return api.list_entities(**object)

@app.route("/entities/<int:id>", methods = ("GET",))
@quorum.ensure("foundation.root_entity.show")
def show_entities(id):
    api = util.get_api()
    entity = api.get_entity(id)
    metadata = entity.get("metadata", None)
    entity["metadata_s"] = _metadata_s(metadata)
    return flask.render_template(
        "entity/show.html.tpl",
        link = "entities",
        sub_link = "info",
        entity = entity
    )

@app.route("/entities/<int:id>/edit", methods = ("GET",))
@quorum.ensure("foundation.root_entity.update")
def edit_entities(id):
    api = util.get_api()
    entity = api.get_entity(id)
    metadata = entity.get("metadata", None)
    entity["metadata_s"] = _metadata_s(metadata)
    return flask.render_template(
        "entity/edit.html.tpl",
        link = "entities",
        sub_link = "edit",
        entity = entity,
        errors = dict()
    )

@app.route("/entities/<int:id>/update", methods = ("POST",))
@quorum.ensure("foundation.root_entity.update")
def update_entities(id):
    models = util.get_models()
    api = util.get_api()
    object = quorum.get_object()
    for name, value in object.items(): object[name] = None if value == "" else value
    entity = models.Entity.new(model = object, build = False, fill = False)
    api.update_entity(id, payload = dict(root_entity = entity.model))
    return flask.redirect(
        flask.url_for("show_entities", id = id)
    )

def _metadata_s(metadata):
    return json.dumps(
        metadata,
        ensure_ascii = False,
        indent = 4,
        separators = (",", " : "),
        sort_keys = True
    ) if not metadata == None else None



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from omnix import util

from omnix.main import app
from omnix.main import flask
from omnix.main import quorum

@app.route("/employees", methods = ("GET",))
@quorum.ensure("foundation.employee.list")
def list_employees():
    return flask.render_template(
        "employee/list.html.tpl",
        link = "employees"
    )

@app.route("/employees.json", methods = ("GET",), json = True)
@quorum.ensure("foundation.employee.list")
def list_employees_json():
    api = util.get_api()
    object = quorum.get_object()
    return api.list_employees(**object)

@app.route("/employees/self", methods = ("GET",))
@quorum.ensure("foundation.employee.show.self")
def show_employee():
    api = util.get_api()
    employee = api.self_employee()
    return flask.render_template(
        "employee/show.html.tpl",
        link = "employees",
        sub_link = "info",
        is_self = True,
        employee = employee
    )

@app.route("/employees/self/sales", methods = ("GET",))
@quorum.ensure(("sales.sale_transaction.list.self", "sales.customer_return.list.self"))
def sales_employee():
    year = quorum.get_field("year", None, cast = int)
    month = quorum.get_field("month", None, cast = int)

    api = util.get_api()
    employee = api.self_employee()

    operations,\
    target,\
    sales_total,\
    sales,\
    returns,\
    previous_month,\
    previous_year,\
    next_month,\
    next_year,\
    has_next = util.get_sales(year = year, month = month)

    return flask.render_template(
        "employee/sales.html.tpl",
        link = "employees",
        sub_link = "sales",
        is_self = True,
        employee = employee,
        operations = operations,
        commission_rate = util.COMMISSION_RATE,
        title = target,
        sales_total = sales_total,
        sales_count = len(sales),
        returns_count = len(returns),
        previous = (previous_month, previous_year),
        next = (next_month, next_year),
        has_next = has_next
    )

@app.route("/employees/self/mail", methods = ("GET",))
@quorum.ensure("foundation.employee.show.self")
def mail_employee():
    year = quorum.get_field("year", None, cast = int)
    month = quorum.get_field("month", None, cast = int)

    util.mail_activity(year = year, month = month)

    return show_employee()

@app.route("/employees/<int:id>", methods = ("GET",))
@quorum.ensure("foundation.employee.show")
def show_employees(id):
    api = util.get_api()
    employee = api.get_employee(id)
    return flask.render_template(
        "employee/show.html.tpl",
        link = "employees",
        sub_link = "info",
        employee = employee
    )

@app.route("/employees/<int:id>/sales", methods = ("GET",))
@quorum.ensure(("sales.sale_transaction.list", "sales.customer_return.list"))
def sales_employees(id):
    year = quorum.get_field("year", None, cast = int)
    month = quorum.get_field("month", None, cast = int)

    api = util.get_api()
    employee = api.get_employee(id)

    operations,\
    target_s,\
    sales_total,\
    sales_s,\
    returns_s,\
    previous_month,\
    previous_year,\
    next_month,\
    next_year,\
    has_next = util.get_sales(id = id, year = year, month = month)

    return flask.render_template(
        "employee/sales.html.tpl",
        link = "employees",
        sub_link = "sales",
        is_self = False,
        employee = employee,
        operations = operations,
        commission_rate = util.COMMISSION_RATE,
        title = target_s,
        sales_total = sales_total,
        sales_count = len(sales_s),
        returns_count = len(returns_s),
        previous = (previous_month, previous_year),
        next = (next_month, next_year),
        has_next = has_next
    )

@app.route("/employees/<int:id>/mail", methods = ("GET",))
@quorum.ensure("foundation.employee.show")
def mail_employees(id):
    year = quorum.get_field("year", None, cast = int)
    month = quorum.get_field("month", None, cast = int)

    util.mail_activity(id = id, year = year, month = month)

    return show_employees(id)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import flask
import quorum

from omnix import util

from . import base

class Settings(base.Base):

    slack_token = quorum.field(
        index = "hashed",
        observations = """The OAuth token from Slack that is going
        to be used for long term Slack interaction after the OAuth
        authentication process is completed"""
    )

    slack_channel = quorum.field(
        index = "hashed"
    )

    extra = quorum.field(
        type = dict
    )

    @classmethod
    def get_settings(cls, *args, **kwargs):
        return cls.singleton(*args, **kwargs)

    @classmethod
    def linked_apis(cls):
        linked = dict()
        settings = cls.get_settings()
        if settings.slack_token: linked["slack"] = settings.slack_token
        return linked

    @classmethod
    def _plural(cls):
        return "Settings"

    def get_slack_api(self):
        try: import slack
        except ImportError: return None
        if not self.slack_token: return None
        redirect_url = util.BASE_URL + flask.url_for("oauth_slack")
        access_token = self.slack_token
        return slack.API(
            client_id = quorum.conf("SLACK_ID"),
            client_secret = quorum.conf("SLACK_SECRET"),
            redirect_url = redirect_url,
            access_token = access_token
        )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import time
import json
import threading

import omni
import quorum

from . import logic
from . import config

LOOP_TIMEOUT = 120
""" The time to be used in between queueing new
messages from the Omni service """

MESSAGE_TIMEOUT = 120
""" The amount of seconds before a message is
considered out dated and is discarded from the
queue even without processing """

RETRY_TIMEOUT = 30
""" The timeout to be used in between the retries
of the operations, as expected """

MESSAGE_RETRIES = 3
""" The number of retries to be used for the message
before it's considered discarded """

NUMBER_RECORDS = config.RECORD_CHUNK
""" The maximum number of records that is going to be
retrieved and set for submission, this value influences
the performance, mostly under heavy load situations """

class Supervisor(threading.Thread):

    session_id = None
    connection = None
    channel = None
    queue = None

    def __init__(self):
        threading.Thread.__init__(self)
        self.daemon = True

    def stop(self):
        pass

    def auth(self):
        if not config.REMOTE: return

        username = config.USERNAME
        password = config.PASSWORD
        if username == None or password == None:
            raise RuntimeError("Missing authentication information")

        self.api = logic.get_api(mode = omni.API.DIRECT_MODE)

    def connect(self, queue = "default", retry = True):
        if not config.REMOTE: return

        quorum.info("Connecting to the AMQP system")

        while True:
            try:
                self.connection = quorum.get_amqp(force = True)
                self.channel = self.connection.channel()
                self.channel.queue_declare(queue = queue, durable = True)
                self.queue = queue
                break
            except Exception as exception:
                if not retry: raise
                quorum.error(
                    "Exception while connecting - %s" % quorum.legacy.UNICODE(exception),
                    log_trace = True
                )
                quorum.info("Sleeping %d seconds before connect retry" % RETRY_TIMEOUT)
                time.sleep(RETRY_TIMEOUT)

    def disconnect(self):
        if not config.REMOTE: return

        quorum.info("Disconnected from the AMQP system")

        self.connection.close()

    def reconnect(self, retry = True):
        if not config.REMOTE: return
        if not self.connection.is_closed: return

        quorum.info("Re-connecting to the AMQP system")

        self.connect(queue = self.queue, retry = retry)

    def execute(self):
        # in case the current instance is not configured according to
        # the remote rules the queuing operation is ignored, and so
        # the control flow returns immediately
        if not config.REMOTE: return

        # creates a values map structure to retrieve the complete
        # set of inbound documents that have not yet been submitted
        # to at for the flush operation
        kwargs = {
            "session_id" : self.session_id,
            "filter_string" : "",
            "start_record" : 0,
            "number_records" : NUMBER_RECORDS,
            "sort" : "issue_date:ascending",
            "filters[]" : [
                "issue_date:greater:1356998400",
                "submitted_at:equals:2",
                "document_type:in:1;3"
            ]
        }
        documents = self.api.list_signed_documents(**kwargs)
        valid_documents = [value for value in documents\
            if value["_class"] in config.AT_SUBMIT_TYPES]

        # starts the counter value to zero, so that we're able to count
        # the number of messages that have been successfully queued to
        # the remote queueing mechanism (for debugging)
        count = 0

        # iterates over all the valid documents that have been found
        # as not submitted and creates a task for their submission
        # then adds the task to the AMQP queue to be processed
        for document in valid_documents:
            try:
                # tries to run the basic publish operation, this operation
                # may fail for a variety of reasons including errors in the
                # underlying library so a reconnection is attempted in case
                # there's an exception raised under this operation
                self.channel.basic_publish(
                    exchange = "",
                    routing_key = config.QUEUE,
                    body = json.dumps(document),
                    properties = quorum.properties_amqp(
                        delivery_mode = 2,
                        priority = MESSAGE_RETRIES,
                        expiration = str(MESSAGE_TIMEOUT * 1000),
                        timestamp = time.time()
                    )
                )
                count += 1
            except Exception as exception:
                # prints a warning message about the exception that has just occurred
                # so that it's possible to act on it
                quorum.warning(
                    "Exception in publish (will re-connect) - %s" % quorum.legacy.UNICODE(exception),
                    log_trace = True
                )

                # re-tries to connect with the AMQP channels using the currently
                # pre-defined queue system, this is a fallback of the error
                self.reconnect()

        # prints an information message about the new documents that
        # have been queued for submission by the "slaves"
        quorum.info("Queued %d (out of %d) documents for submission" % (count, len(valid_documents)))

    def loop(self):
        while True:
            try: self.execute()
            except Exception as exception:
                # prints an error message about the exception that has just occurred
                # so that it's possible to act on it
                quorum.error(
                    "Exception while executing - %s" % quorum.legacy.UNICODE(exception),
                    log_trace = True
                )

                # re-tries to connect with the AMQP channels using the currently
                # pre-defined queue system, this is a fallback of the error
                self.reconnect()

            try:
                if self.connection: self.connection.sleep(LOOP_TIMEOUT)
                else: time.sleep(LOOP_TIMEOUT)
            except Exception as exception:
                # prints a critical message about the exception that has just occurred
                # so that it's possible to act on it
                quorum.critical(
                    "Exception while sleeping - %s" % quorum.legacy.UNICODE(exception),
                    log_trace = True
                )

                # re-tries to connect with the AMQP channels using the currently
                # pre-defined queue system, this is a fallback of the error
                self.reconnect()

    def run(self):
        self.auth()
        self.connect(queue = config.QUEUE)
        try: self.loop()
        finally: self.disconnect()

def run(count = 1):
    for _index in range(count):
        supervisor = Supervisor()
        supervisor.start()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import datetime

from omnix import util

from omnix.main import app
from omnix.main import flask
from omnix.main import quorum

@app.route("/stores", methods = ("GET",))
@quorum.ensure("foundation.store.list")
def list_stores():
    return flask.render_template(
        "store/list.html.tpl",
        link = "stores"
    )

@app.route("/stores.json", methods = ("GET",), json = True)
@quorum.ensure("foundation.store.list")
def list_stores_json():
    api = util.get_api()
    object = quorum.get_object()
    return api.list_stores(**object)

@app.route("/stores/<id>", methods = ("GET",))
@quorum.ensure("foundation.store.show")
def show_stores(id):
    api = util.get_api()
    id = int(id)
    is_global = id == -1
    store = _global() if is_global else api.get_store(id)
    return flask.render_template(
        "store/show.html.tpl",
        link = "stores",
        sub_link = "info",
        store = store
    )

@app.route("/stores/<id>/sales", methods = ("GET",))
@quorum.ensure(("foundation.store.show", "analytics.sale_snapshot.list"))
def sales_stores(id):
    api = util.get_api()

    id = int(id)
    is_global = id == -1

    now = datetime.datetime.utcnow()
    current_day = datetime.datetime(now.year, now.month, now.day)

    store = _global() if is_global else api.get_store(id)

    contents = api.stats_sales(
        unit = "day",
        store_id = None if is_global else id,
        has_global = True
    )
    stats = contents[str(id)]
    current = dict(
        net_price_vat = stats["net_price_vat"][-1],
        net_number_sales = stats["net_number_sales"][-1],
        date = current_day
    )

    days = []

    count = len(stats["net_price_vat"]) - 1
    count_r = range(count)
    count_r = list(count_r)
    count_r.reverse()
    _current_day = current_day
    for index in count_r:
        _current_day -= datetime.timedelta(1)
        day = dict(
            net_price_vat = stats["net_price_vat"][index],
            net_number_sales = stats["net_number_sales"][index],
            date = _current_day
        )
        days.append(day)

    previous_s = days[0] if days else dict()
    current["amount_delta"] = current["net_price_vat"] -\
        previous_s.get("net_price_vat", 0)
    current["number_delta"] = current["net_number_sales"] -\
        previous_s.get("net_number_sales", 0)

    if current["amount_delta"] == 0: current["amount_direction"] = "equal"
    elif current["amount_delta"] > 0: current["amount_direction"] = "up"
    else: current["amount_direction"] = "down"

    if current["number_delta"] == 0: current["number_direction"] = "equal"
    elif current["number_delta"] > 0: current["number_direction"] = "up"
    else: current["number_direction"] = "down"

    return flask.render_template(
        "store/sales.html.tpl",
        link = "stores",
        sub_link = "sales",
        store = store,
        stats = stats,
        current = current,
        days = days
    )

def _global():
    return dict(
        object_id = -1,
        name = "Global",
        primary_contact_information = dict()
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import quorum

from . import config

def mask_image(base_data, mask_data, format = "png"):
    import PIL.Image

    base_file = quorum.legacy.BytesIO(base_data)
    mask_file = quorum.legacy.BytesIO(mask_data)
    out_file = quorum.legacy.BytesIO()

    base_image = PIL.Image.open(base_file)
    mask_image = PIL.Image.open(mask_file)

    target_image = PIL.Image.new("RGBA", mask_image.size, (255, 255, 255, 255))

    base_width, base_height = base_image.size
    mask_width, mask_height = mask_image.size

    base_image = base_image.convert("RGBA")

    if config.IMAGE_RESIZE == "stretch":
        base_image = base_image.resize(mask_image.size, PIL.Image.ANTIALIAS)
    elif config.IMAGE_RESIZE == "box":
        ratio_width = float(mask_width) / float(base_width)
        ratio_height = float(mask_height) / float(base_height)

        ratio = ratio_width if ratio_width < ratio_height else ratio_height
        resize_width = int(base_width * ratio)
        resize_height = int(base_height * ratio)

        base_image = base_image.resize(
            (resize_width, resize_height),
            PIL.Image.ANTIALIAS
        )
    elif config.IMAGE_RESIZE == "crop":
        ratio_width = float(mask_width) / float(base_width)
        ratio_height = float(mask_height) / float(base_height)

        ratio = ratio_height if ratio_width < ratio_height else ratio_width
        resize_width = int(base_width * ratio)
        resize_height = int(base_height * ratio)

        base_image = base_image.resize(
            (resize_width, resize_height),
            PIL.Image.ANTIALIAS
        )

        center_width = (resize_width - mask_width) / 2.0
        center_height = (resize_height - mask_height) / 2.0
        center_width = int(center_width)
        center_height = int(center_height)

        crop_box = (
            center_width,
            center_height,
            mask_width + center_width,
            mask_height + center_height
        )
        base_image = base_image.crop(crop_box)

    base_width, base_height = base_image.size
    center_width = (mask_width - base_width) / 2.0
    center_height = (mask_height - base_height) / 2.0
    center = (int(center_width), int(center_height))

    target_image.paste(base_image, center, base_image)
    target_image.paste(mask_image, (0, 0), mask_image)
    target_image.save(out_file, format)

    out_data = out_file.getvalue()

    return out_data



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import base
from . import customer
from . import employee
from . import entity
from . import extra
from . import media
from . import report
from . import settings
from . import store
from . import supplier

from .base import index, signin, login, do_login, logout, about, reset, flush_slack_sales,\
    flush_slack_previous, flush_slack_week, flush_birthday, flush_activity, flush_at,\
    oauth, top, handler_404, handler_413, handler_exception
from .customer import list_customers, list_customers_json, show_customers
from .employee import list_employees, list_employees_json, show_employee, sales_employee,\
    mail_employee, show_employees, sales_employees, mail_employees
from .entity import list_entities, list_entities_json, show_entities, edit_entities
from .extra import list_extras, media_extras, do_media_extras, images_extras, do_images_extras,\
    prices_extras, do_prices_extras, inventory_extras, do_inventory_extras, template_extras,\
    do_template_extras, do_mask_extras, browser_extras, do_browser, new_media_browser,\
    create_media_browser
from .media import list_media, list_media_json, show_media, edit_media, update_media,\
    delete_media
from .report import list_reports, sales_reports
from .settings import oauth_slack
from .store import list_stores, list_stores_json, show_stores, sales_stores
from .supplier import list_suppliers, list_suppliers_json, show_suppliers



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import calendar
import datetime
import unittest

import omnix

class BusinessTest(unittest.TestCase):

    def test_get_comparison_day(self):
        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 2, day = 28, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 2, day = 28, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "day", offset = -1, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 28)
        self.assertEqual(current_v["unit"], "day")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 28)
        self.assertEqual(previous_v["unit"], "day")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 1, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 3, day = 1, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "day", offset = 0, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 1)
        self.assertEqual(current_v["unit"], "day")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 1)
        self.assertEqual(previous_v["unit"], "day")
        self.assertEqual(previous_v["has_global"], True)

    def test_get_comparison_day_leap(self):
        timestamp = calendar.timegm(datetime.datetime(year = 2020, month = 3, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2020, month = 2, day = 29, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2019, month = 2, day = 28, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "day", offset = -1, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 29)
        self.assertEqual(current_v["unit"], "day")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 28)
        self.assertEqual(previous_v["unit"], "day")
        self.assertEqual(previous_v["has_global"], True)

    def test_get_comparison_month(self):
        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 2, day = 28, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 2, day = 28, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -1, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 2)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 2)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 1, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 3, day = 1, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = 0, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 3)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 3)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 2, day = 25, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 2, day = 25, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -4, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 2)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 2)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 4, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 3, day = 31, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 3, day = 31, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -1, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 3)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 3)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 1, day = 2, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 1, day = 1, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 1, day = 1, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -1, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 1)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 1)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 2, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 1, day = 31, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 1, day = 31, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -1, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 1)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 1)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 2, day = 2, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2018, month = 1, day = 31, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2017, month = 1, day = 31, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -2, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 1)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 1)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

        timestamp = calendar.timegm(datetime.datetime(year = 2018, month = 1, day = 2, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2017, month = 12, day = 31, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2016, month = 12, day = 31, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -2, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 12)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 12)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)

    def test_get_comparison_month_leap(self):
        timestamp = calendar.timegm(datetime.datetime(year = 2020, month = 3, day = 1, hour = 4).utctimetuple())
        current = calendar.timegm(datetime.datetime(year = 2020, month = 2, day = 29, hour = 4).utctimetuple())
        previous = calendar.timegm(datetime.datetime(year = 2019, month = 2, day = 28, hour = 4).utctimetuple())
        current_v, previous_v = omnix.calc_comparison(unit = "month", offset = -1, timestamp = timestamp)

        self.assertEqual(current_v["date"], current)
        self.assertEqual(current_v["span"], 2)
        self.assertEqual(current_v["unit"], "month")
        self.assertEqual(current_v["has_global"], True)

        self.assertEqual(previous_v["date"], previous)
        self.assertEqual(previous_v["span"], 2)
        self.assertEqual(previous_v["unit"], "month")
        self.assertEqual(previous_v["has_global"], True)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import mimetypes

from omnix import util

from omnix.main import app
from omnix.main import flask
from omnix.main import quorum

@app.route("/media", methods = ("GET",))
@quorum.ensure("foundation.media.list")
def list_media():
    return flask.render_template(
        "media/list.html.tpl",
        link = "media"
    )

@app.route("/media.json", methods = ("GET",), json = True)
@quorum.ensure("foundation.media.list")
def list_media_json():
    api = util.get_api()
    object = quorum.get_object()
    return api.list_media(**object)

@app.route("/media/<int:id>", methods = ("GET",))
@quorum.ensure("foundation.media.show")
def show_media(id):
    api = util.get_api()
    media = api.info_media(id)
    media["image_url"] = api.get_media_url(media["secret"])
    return flask.render_template(
        "media/show.html.tpl",
        link = "media",
        sub_link = "info",
        media = media
    )

@app.route("/media/<int:id>/edit", methods = ("GET",))
@quorum.ensure("foundation.media.update")
def edit_media(id):
    api = util.get_api()
    media = api.info_media(id)
    return flask.render_template(
        "media/edit.html.tpl",
        link = "media",
        sub_link = "edit",
        media = media,
        errors = dict()
    )

@app.route("/media/<int:id>/update", methods = ("POST",))
@quorum.ensure("foundation.media.update")
def update_media(id):
    api = util.get_api()
    position = quorum.get_field("position", None, cast = int)
    label = quorum.get_field("label", None) or None
    visibility = quorum.get_field("visibility", None, cast = int)
    description = quorum.get_field("description", None) or None
    media_file = quorum.get_field("media_file", None)
    image_type = mimetypes.guess_type(media_file.filename)[0]
    mime_type = image_type if image_type else "image/unknown"
    media = dict(
        position = position,
        label = label,
        visibility = visibility,
        description = description
    )
    payload = dict(media = media)
    if media_file:
        try: data = media_file.stream.read()
        finally: media_file.close()
        media["mime_type"] = mime_type
        payload["data"] = data
    media = api.update_media(id, payload)
    return flask.redirect(
        flask.url_for("show_media", id = media["object_id"])
    )

@app.route("/media/<int:id>/delete", methods = ("GET",))
@quorum.ensure("foundation.media.delete")
def delete_media(id):
    api = util.get_api()
    api.delete_media(id)
    return flask.redirect(
        flask.url_for("list_media")
    )



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import flask #@UnusedImport
import datetime

import quorum

import omnix.models

MONGO_DATABASE = "omnix"
""" The default database to be used for the connection with
the MongoDB database """

SECRET_KEY = "zhsga32ki5kvv7ymq8nolbleg248fzn1"
""" The "secret" key to be at the internal encryption
processes handled by flask (eg: sessions) """

@quorum.onrun
def onrun():
    import omnix.util
    omnix.util.run_slave(1)
    omnix.util.run_supervisor()
    omnix.util.load_scheduling()

app = quorum.load(
    name = __name__,
    secret_key = SECRET_KEY,
    mongo_database = MONGO_DATABASE,
    logger = "omnix.debug",
    models = omnix.models,
    PERMANENT_SESSION_LIFETIME = datetime.timedelta(31),
    MAX_CONTENT_LENGTH = 1024 ** 3
)

import omnix.views #@UnusedImport

if __name__ == "__main__":
    quorum.run(server = "netius")
else:
    __path__ = []



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

from . import business
from . import config
from . import ctt
from . import format
from . import image
from . import logic
from . import scheduling
from . import slave
from . import supervisor

from .business import slack_sales, slack_previous, slack_week, mail_birthday_all,\
    mail_activity_all, mail_birthday, mail_activity, get_date, get_top, get_sales,\
    get_comparison, sum_results, calc_comparison, calc_extra, calc_results
from .ctt import encode_ctt
from .format import csv_file, csv_import, csv_value
from .config import LOCAL_PREFIX, REMOTE_PREFIX, LOCAL_URL, REMOTE_URL, REDIRECT_URL,\
    CLIENT_ID, CLIENT_SECRET, FIRST_DAY, SCOPE, AT_SALE_TYPES, AT_TRANSPORT_TYPES,\
    AT_SUBMIT_TYPES, REMOTE, BASE_URL, SENDER_EMAIL, USERNAME, PASSWORD, SCHEDULE,\
    COMMISSION_RATE, COMMISSION_DAY, IMAGE_RESIZE, LOCALE, QUEUE, RECORD_CHUNK,\
    BIRTHDAY_TEMPLATE, OMNI_URL, PREFIX
from .image import mask_image
from .logic import get_models, get_api, ensure_api, on_auth, start_session, reset_session,\
    get_tokens

from .scheduling import load as load_scheduling
from .slave import run as run_slave
from .supervisor import run as run_supervisor



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

def encode_ctt(sale_orders, encoding = None):
    lines = []

    for sale_order in sale_orders:
        line = dict(
            reference = sale_order["extended_identifier"][:21],
            quantity = 1,
            weight = "100,00",
            price = "0ue",
            destiny = sale_order["customer"]["representation"][:60],
            title = "",
            name = "",
            address = sale_order["shipping_address"]["street_name"][:60],
            town = sale_order["shipping_address"]["zip_code_name"][:50],
            zip_code_4 = sale_order["shipping_address"]["zip_code"].split("-", 1)[0][:4],
            zip_code_3 = sale_order["shipping_address"]["zip_code"].split("-", 1)[1][:3],
            not_applicable_1 = "",
            observations = "",
            back = 0,
            document_code = "",
            phone_number = (sale_order["customer"]["primary_contact_information"]["phone_number"] or "").replace("+", "00")[:15],
            saturday = 0,
            email = (sale_order["customer"]["primary_contact_information"]["email"] or "")[:200],
            country = "PT",
            fragile = 0,
            not_applicable_2 = "",
            document_collection = "",
            code_email = "",
            mobile_phone = (sale_order["customer"]["primary_contact_information"]["mobile_phone_number"] or "").replace("+", "00")[:15],
            second_delivery = 0,
            delivery_date = "",
            return_signed_document = 0,
            expeditor_instructions = 0,
            sms = 1,
            not_applicable_3 = "",
            printer = "",
            ticket_machine = "",
            at_code = ""
        )

        line = "+".join([
            line["reference"],
            str(line["quantity"]),
            line["weight"],
            line["price"],
            line["destiny"],
            line["title"],
            line["name"],
            line["address"],
            line["town"],
            line["zip_code_4"],
            line["zip_code_3"],
            line["not_applicable_1"],
            line["observations"],
            str(line["back"]),
            line["document_code"],
            line["phone_number"],
            str(line["saturday"]),
            line["email"],
            line["country"],
            str(line["fragile"]),
            line["not_applicable_2"],
            line["document_collection"],
            line["code_email"],
            line["mobile_phone"],
            str(line["second_delivery"]),
            line["delivery_date"],
            str(line["return_signed_document"]),
            str(line["expeditor_instructions"]),
            str(line["sms"]),
            line["not_applicable_3"],
            line["printer"],
            line["ticket_machine"],
            line["at_code"]
        ])

        lines.append(line)

    ctt_data = "\n".join(lines)
    if encoding: ctt_data = ctt_data.encode(encoding)

    return ctt_data



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import os
import setuptools

setuptools.setup(
    name = "omnix",
    version = "0.2.4",
    author = "Hive Solutions Lda.",
    author_email = "development@hive.pt",
    description = "Omnix System",
    license = "Apache License, Version 2.0",
    keywords = "omni extensions erp",
    url = "http://omnix.hive.pt",
    zip_safe = False,
    packages = [
        "omnix",
        "omnix.models",
        "omnix.test",
        "omnix.test.util",
        "omnix.util",
        "omnix.views",
        "omnix.views.api",
        "omnix.views.web"
    ],
    test_suite = "omnix.test",
    package_dir = {
        "" : os.path.normpath("src")
    },
    package_data = {
        "omnix" : [
            "static/css/*.css",
            "static/images/*.png",
            "static/images/email/*.png",
            "static/js/*.js",
            "templates/*.tpl",
            "templates/customer/*.tpl",
            "templates/email/*.tpl",
            "templates/employee/*.tpl",
            "templates/entity/*.tpl",
            "templates/extra/*.tpl",
            "templates/extra/browser/*.tpl",
            "templates/media/*.tpl",
            "templates/partials/*.tpl",
            "templates/report/*.tpl",
            "templates/store/*.tpl",
            "templates/supplier/*.tpl"
        ]
    },
    install_requires = [
        "netius",
        "flask",
        "quorum",
        "pillow",
        "pymongo",
        "redis",
        "pika",
        "xlrd",
        "omni_api"
    ],
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Topic :: Utilities",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.0",
        "Programming Language :: Python :: 3.1",
        "Programming Language :: Python :: 3.2",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7"
    ]
)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Hive Omnix System
# Copyright (c) 2008-2022 Hive Solutions Lda.
#
# This file is part of Hive Omnix System.
#
# Hive Omnix System is free software: you can redistribute it and/or modify
# it under the terms of the Apache License as published by the Apache
# Foundation, either version 2.0 of the License, or (at your option) any
# later version.
#
# Hive Omnix System is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# Apache License for more details.
#
# You should have received a copy of the Apache License along with
# Hive Omnix System. If not, see <http://www.apache.org/licenses/>.

__author__ = "João Magalhães <joamag@hive.pt>"
""" The author(s) of the module """

__version__ = "1.0.0"
""" The version of the module """

__revision__ = "$LastChangedRevision$"
""" The revision number of the module """

__date__ = "$LastChangedDate$"
""" The last change date of the module """

__copyright__ = "Copyright (c) 2008-2022 Hive Solutions Lda."
""" The copyright for the module """

__license__ = "Apache License, Version 2.0"
""" The license for the module """

import omni

import flask

import quorum

from . import config

def get_models():
    return omni.models

def get_api(mode = omni.API.OAUTH_MODE):
    access_token = flask.session and flask.session.get("omnix.access_token", None)
    session_id = flask.session and flask.session.get("omnix.session_id", None)
    api = omni.API(
        base_url = config.OMNI_URL,
        open_url = config.OMNI_URL,
        prefix = config.PREFIX,
        client_id = config.CLIENT_ID,
        client_secret = config.CLIENT_SECRET,
        redirect_url = config.REDIRECT_URL,
        scope = config.SCOPE,
        access_token = access_token,
        session_id = session_id,
        username = config.USERNAME,
        password = config.PASSWORD,
        mode = mode
    )
    api.bind("auth", on_auth)
    return api

def ensure_api(state = None):
    access_token = flask.session.get("omnix.access_token", None)
    if access_token: return
    api = get_api()
    return api.oauth_authorize(state = state)

def on_auth(contents):
    start_session(contents)

def start_session(contents):
    if not flask.session: return

    username = contents.get("username", None)
    acl = contents.get("acl", None)
    session_id = contents.get("session_id", None)
    tokens = get_tokens(acl)

    flask.session["omnix.base_url"] = config.OMNI_URL
    flask.session["omnix.username"] = username
    flask.session["omnix.acl"] = acl
    flask.session["omnix.session_id"] = session_id
    flask.session["tokens"] = tokens

def reset_session():
    if not flask.session: return

    if "omnix.base_url" in flask.session: del flask.session["omnix.base_url"]
    if "omnix.access_token" in flask.session: del flask.session["omnix.access_token"]
    if "omnix.username" in flask.session: del flask.session["omnix.username"]
    if "omnix.acl" in flask.session: del flask.session["omnix.acl"]
    if "omnix.session_id" in flask.session: del flask.session["omnix.session_id"]
    if "tokens" in flask.session: del flask.session["tokens"]
    flask.session.modified = True

def get_tokens(acl):
    tokens = acl.keys()
    return quorum.legacy.eager(tokens)
